#!/usr/bin/env python
# pylint:disable=W0201,W0613,R0201,W0212

"""Unit tests for the `base` module."""

import pytest
import logging

from yorm.base import Mappable, Converter
from yorm.mapper import Mapper
from yorm.standard import String, Integer, Boolean

from . import strip


class MockMapper(Mapper):

    """Mapped file with stubbed file IO."""

    def __init__(self, path):
        super().__init__(path)
        self._mock_file = None
        self._mock_modified = True
        self.exists = True

    def _read(self):
        text = self._mock_file
        logging.debug("mock read:\n%s", text.strip())
        return text

    def _write(self, text):
        logging.debug("mock write:\n%s", text.strip())
        self._mock_file = text
        self.modified = True

    @property
    def modified(self):
        return self._mock_modified

    @modified.setter
    def modified(self, changes):  # pylint: disable=W0221
        self._mock_modified = changes


# sample classes ##############################################################


class SampleMappable(Mappable):

    """Sample mappable class with hard-coded settings."""

    def __init__(self):
        logging.debug("initializing sample...")
        self.var1 = None
        self.var2 = None
        self.var3 = None
        logging.debug("sample initialized")

        self.yorm_path = "mock/path/to/sample.yml"
        self.yorm_attrs = {'var1': String,
                           'var2': Integer,
                           'var3': Boolean}
        self.yorm_mapper = MockMapper(self.yorm_path)
        self.yorm_mapper.store(self, self.yorm_attrs)
        self.yorm_mapper.auto = True

    def __repr__(self):
        return "<sample {}>".format(id(self))


# tests #######################################################################


class TestMappable:

    """Unit tests for the `Mappable` class."""

    def setup_method(self, method):
        """Create an mappable instance for tests."""
        self.sample = SampleMappable()

    def test_init(self):
        """Verify files are created after initialized."""
        text = self.sample.yorm_mapper._read()
        assert strip("""
        var1: ''
        var2: 0
        var3: false
        """) == text

    def test_set(self):
        """Verify the file is written to after setting an attribute."""
        self.sample.var1 = "abc123"
        self.sample.var2 = 1
        self.sample.var3 = True
        text = self.sample.yorm_mapper._read()
        assert strip("""
        var1: abc123
        var2: 1
        var3: true
        """) == text

    def test_set_converted(self):
        """Verify conversion occurs when setting attributes."""
        self.sample.var1 = 42
        self.sample.var2 = "1"
        self.sample.var3 = 'off'
        text = self.sample.yorm_mapper._read()
        assert strip("""
        var1: '42'
        var2: 1
        var3: false
        """) == text

    def test_set_error(self):
        """Verify an exception is raised when a value cannot be converted."""
        with pytest.raises(ValueError):
            self.sample.var2 = "abc"

    def test_get(self):
        """Verify the file is read from before getting an attribute."""
        text = strip("""
        var1: def456
        var2: 42
        var3: off
        """)
        self.sample.yorm_mapper._write(text)
        assert"def456" == self.sample.var1
        assert 42 == self.sample.var2
        assert False is self.sample.var3

    def test_error_invalid_yaml(self):
        """Verify an exception is raised on invalid YAML."""
        text = strip("""
        invalid: -
        """)
        self.sample.yorm_mapper._write(text)
        with pytest.raises(ValueError):
            print(self.sample.var1)

    def test_error_unexpected_yaml(self):
        """Verify an exception is raised on unexpected YAML."""
        text = strip("""
        not a dictionary
        """)
        self.sample.yorm_mapper._write(text)
        with pytest.raises(ValueError):
            print(self.sample.var1)

    def test_context_manager(self):
        """Verify the context manager delays write."""
        with self.sample:
            self.sample.var1 = "abc123"

            text = self.sample.yorm_mapper._read()
            assert strip("""
            var1: ''
            var2: 0
            var3: false
            """) == text

        text = self.sample.yorm_mapper._read()
        assert strip("""
        var1: abc123
        var2: 0
        var3: false
        """) == text

    def test_new(self):
        """Verify new attributes are added to the object."""
        text = strip("""
        new: 42
        """)
        self.sample.yorm_mapper._write(text)
        assert 42 == self.sample.new

    def test_new_unknown(self):
        """Verify an exception is raised on new attributes w/ unknown types"""
        text = strip("""
        new: !!timestamp 2001-12-15T02:59:43.1Z
        """)
        self.sample.yorm_mapper._write(text)
        with pytest.raises(ValueError):
            print(self.sample.var1)


class TestConverter:

    """Unit tests for the `Converter` class."""

    def test_not_implemented(self):
        """Verify `Converter` cannot be used directly."""
        with pytest.raises(NotImplementedError):
            Converter.to_value(None)  # pylint: disable=E1120
        with pytest.raises(NotImplementedError):
            Converter.to_data(None)  # pylint: disable=E1120


if __name__ == '__main__':
    pytest.main()



"""Common exceptions, classes, and functions."""

import os
import shutil
import logging

import yaml

verbosity = 0  # global verbosity setting for controlling string formatting
PRINT_VERBOSITY = 0  # minimum verbosity to using `print`
STR_VERBOSITY = 3  # minimum verbosity to use verbose `__str__`
MAX_VERBOSITY = 4  # maximum verbosity level implemented


def _trace(self, message, *args, **kws):  # pragma: no cover (manual test)
    """Handler for a new TRACE logging level."""
    if self.isEnabledFor(logging.DEBUG - 1):
        self._log(logging.DEBUG - 1, message, args, **kws)  # pylint: disable=W0212


logging.addLevelName(logging.DEBUG - 1, "TRACE")
logging.Logger.trace = _trace

logger = logging.getLogger
log = logger(__name__)


# exception classes ##########################################################


class YORMException(Exception):

    """Base class for all YORM exceptions."""


class FileError(YORMException, FileNotFoundError):

    """Raised when text cannot be read from a file."""


class ContentError(YORMException, yaml.error.YAMLError, ValueError):

    """Raised when YAML cannot be parsed from text."""


class ConversionError(YORMException, ValueError):

    """Raised when a value cannot be converted to the specified type."""


class UseageError(YORMException):

    """Raised when an API is called incorrectly."""

# decorators #################################################################


class classproperty(object):

    """Read-only class property decorator."""

    def __init__(self, getter):
        self.getter = getter

    def __get__(self, instance, owner):
        return self.getter(owner)


# disk helper functions ######################################################


def create_dirname(path):
    """Ensure a parent directory exists for a path."""
    dirpath = os.path.dirname(path)
    if dirpath and not os.path.isdir(dirpath):
        log.trace("creating directory '{}'...".format(dirpath))
        os.makedirs(dirpath)


def read_text(path, encoding='utf-8'):
    """Read text from a file.

    :param path: file path to read from
    :param encoding: input file encoding

    :return: string

    """
    log.trace("reading text from '{}'...".format(path))
    with open(path, 'r', encoding=encoding) as stream:
        text = stream.read()
    return text


def load_yaml(text, path):
    """Parse a dictionary from YAML text.

    :param text: string containing dumped YAML data
    :param path: file path for error messages

    :return: dictionary

    """
    # Load the YAML data
    try:
        data = yaml.load(text) or {}
    except yaml.error.YAMLError as exc:
        msg = "invalid contents: {}:\n{}".format(path, exc)
        raise ContentError(msg) from None
    # Ensure data is a dictionary
    if not isinstance(data, dict):
        msg = "invalid contents: {}".format(path)
        raise ContentError(msg)
    # Return the parsed data
    return data


def write_text(text, path, encoding='utf-8'):
    """Write text to a file.

    :param text: string
    :param path: file to write text
    :param encoding: output file encoding

    :return: path of new file

    """
    if text:
        log.trace("writing text to '{}'...".format(path))
    with open(path, 'wb') as stream:
        data = text.encode(encoding)
        stream.write(data)
    return path


def touch(path):
    """Ensure a file exists."""
    if not os.path.exists(path):
        dirpath = os.path.dirname(path)
        if dirpath and not os.path.isdir(dirpath):
            log.trace("creating directory '{}'...".format(dirpath))
            os.makedirs(dirpath)
        log.trace("creating empty '{}'...".format(path))
        write_text('', path)


def stamp(path):
    """Get the modification timestamp from a file."""
    return os.path.getmtime(path)


def delete(path):
    """Delete a file or directory with error handling."""
    if os.path.isdir(path):
        try:
            log.trace("deleting '{}'...".format(path))
            shutil.rmtree(path)
        except IOError:  # pragma: no cover (manual test)
            # bug: http://code.activestate.com/lists/python-list/159050
            msg = "unable to delete: {}".format(path)
            log.warning(msg)
    elif os.path.isfile(path):
        log.trace("deleting '{}'...".format(path))
        os.remove(path)



#!/usr/bin/env python
# pylint:disable=R0201

"""Integration tests for the `common` module."""

import os
import pytest

from yorm import common


def test_file_creation(tmpdir):
    """Verify a file can be created in an existing directory."""
    tmpdir.chdir()

    path = os.path.join('.', 'file.ext')

    common.touch(path)
    assert os.path.isfile(path)

    common.touch(path)  # second call is ignored
    assert os.path.isfile(path)


def test_file_creation_nested(tmpdir):
    """Verify a file can be created in a new directory."""
    tmpdir.chdir()

    dirpath = os.path.join('path', 'to', 'directory')
    path = os.path.join(dirpath, 'file.ext')

    common.touch(path)
    assert os.path.isfile(path)


def test_file_deletion(tmpdir):
    """Verify a file can be deleted."""
    tmpdir.chdir()

    path = os.path.join('.', 'file.ext')

    common.touch(path)
    assert os.path.isfile(path)

    common.delete(path)
    assert not os.path.isfile(path)

    common.delete(path)  # second call is ignored
    assert not os.path.isfile(path)


def test_directory_deletion(tmpdir):
    """Verify a directory can be deleted."""
    tmpdir.chdir()

    dirpath = os.path.join('path', 'to', 'directory')
    path = os.path.join(dirpath, 'file.ext')

    common.touch(path)
    assert os.path.isdir(dirpath)

    common.delete(dirpath)
    assert not os.path.isdir(dirpath)

    common.delete(dirpath)  # second call is ignored
    assert not os.path.isdir(dirpath)


if __name__ == '__main__':
    pytest.main()



#!/usr/bin/env python
# pylint:disable=R0201

"""Unit tests for the `extended` module."""

import pytest

from yorm.utilities import attr
from yorm.extended import Markdown, AttributeDictionary, SortedList
from yorm.standard import Integer, String, Float


# sample classes ##############################################################


@attr(var1=Integer, var2=String)
class SampleAttributeDictionary(AttributeDictionary):

    """Sample dictionary container with initialization."""

    def __init__(self, var1, var2, var3):
        super().__init__()
        self.var1 = var1
        self.var2 = var2
        self.var3 = var3


@attr(all=Float)
class SampleSortedList(SortedList):

    """Sample sorted list container."""


class UnknownSortedList(SortedList):

    """Sample list container."""


# tests #######################################################################


class TestMarkdown:

    """Unit tests for the `Markdown` converter."""

    obj = "This is **the** sentence."

    data_value = [
        (obj, obj),
        (None, ""),
        (['a', 'b', 'c'], "a, b, c"),
        ("This is\na sentence.", "This is a sentence."),
        ("Sentence one.\nSentence two.", "Sentence one. Sentence two."),
    ]

    value_data = [
        (obj, obj + '\n'),
        ("Sentence one. Sentence two.", "Sentence one.\nSentence two.\n"),
        ("", ""),
        (" \t ", ""),
    ]

    @pytest.mark.parametrize("data,value", data_value)
    def test_to_value(self, data, value):
        """Verify input data is converted to values."""
        assert value == Markdown.to_value(data)

    @pytest.mark.parametrize("value,data", value_data)
    def test_to_data(self, value, data):
        """Verify values are converted to output data."""
        assert data == Markdown.to_data(value)


class TestAttributeDictionary:

    """Unit tests for the `AttributeDictionary` container."""

    def test_not_implemented(self):
        """Verify `AttributeDictionary` cannot be used directly."""
        with pytest.raises(NotImplementedError):
            AttributeDictionary.to_value(None)
        with pytest.raises(NotImplementedError):
            AttributeDictionary.to_data(None)

    def test_attribute_access(self):
        """Verify `AttributeDictionary` keys are available as attributes."""
        obj = SampleAttributeDictionary(1, 2, 3.0)
        value = {'var1': 1, 'var2': '2'}
        value2 = obj.to_value(obj)
        assert value == value2
        assert 1 == value2.var1
        assert '2' == value2.var2
        assert not hasattr(value2, 'var3')  # lost in conversion


class TestSortedList:

    """Unit tests for the `SortedList` container."""

    def test_not_implemented(self):
        """Verify `SortedList` cannot be used directly."""
        with pytest.raises(NotImplementedError):
            SortedList.to_value(None)
        with pytest.raises(NotImplementedError):
            SortedList.to_data(None)
        with pytest.raises(NotImplementedError):
            UnknownSortedList.to_value(None)
        with pytest.raises(NotImplementedError):
            UnknownSortedList.to_data(None)

    def test_sorted_result(self):
        """Verify `SortedList` sorts the resulting data."""
        obj = SampleSortedList([4, 2, 0, 1, 3])
        data = [0.0, 1.0, 2.0, 3.0, 4.0]
        data2 = obj.to_data(obj)
        assert data == data2


if __name__ == '__main__':
    pytest.main()



#!/usr/bin/env python

"""Tests for the `yorm` package."""


def strip(text, tabs=None, end='\n'):
    """Strip leading whitespace indentation on multiline string literals."""
    lines = text.strip().splitlines()
    for index in range(len(lines)):
        if not tabs:
            tabs = lines[index].count(' ' * 4)
        lines[index] = lines[index].replace(' ' * tabs * 4, '')
    return '\n'.join(lines) + end



"""Package for YORM."""

import sys

__project__ = 'YORM'
__version__ = '0.3.1'

VERSION = __project__ + '-' + __version__

PYTHON_VERSION = 3, 3

if not sys.version_info >= PYTHON_VERSION:  # pragma: no cover (manual test)
    exit("Python {}.{}+ is required.".format(*PYTHON_VERSION))

try:
    from . import standard, extended, container
    from .utilities import UUID
    from .utilities import sync, sync_object, sync_instances, attr
    from .utilities import update, update_object, update_file
    from .base import Mappable, Converter
except ImportError:  # pragma: no cover (manual test)
    pass



"""Base classes."""

import abc

from . import common

log = common.logger(__name__)


class Mappable(metaclass=abc.ABCMeta):

    """Base class for objects with attributes that map to YAML."""

    def __getattribute__(self, name):
        if name.startswith('yorm_'):
            return object.__getattribute__(self, name)

        try:
            value = object.__getattribute__(self, name)
        except AttributeError:
            self.yorm_mapper.fetch(self, self.yorm_attrs)
            value = object.__getattribute__(self, name)
        else:
            if name in self.yorm_attrs:
                self.yorm_mapper.fetch(self, self.yorm_attrs)
                value = object.__getattribute__(self, name)

        return value

    def __setattr__(self, name, value):
        if hasattr(self, 'yorm_attrs') and name in self.yorm_attrs:
            converter = self.yorm_attrs[name]
            value = converter.to_value(value)

        object.__setattr__(self, name, value)

        if hasattr(self, 'yorm_attrs') and name in self.yorm_attrs:
            if hasattr(self, 'yorm_mapper') and self.yorm_mapper.auto:
                self.yorm_mapper.store(self, self.yorm_attrs)
            else:
                log.trace("automatic storage is off")

    def __enter__(self):
        log.debug("turning off automatic storage...")
        self.yorm_mapper.auto = False

    def __exit__(self, *_):
        log.debug("turning on automatic storage...")
        self.yorm_mapper.store(self, self.yorm_attrs)


class Converter(metaclass=abc.ABCMeta):

    """Base class for attribute converters."""

    TYPE = None  # type for inferred converters (set in subclasses)
    DEFAULT = None  # default value for conversion (set in subclasses)

    @abc.abstractclassmethod
    def to_value(cls, obj):  # pylint: disable=E0213
        """Convert the loaded value back to its original attribute type."""
        raise NotImplementedError("method must be implemented in subclasses")

    @abc.abstractclassmethod
    def to_data(cls, obj):  # pylint: disable=E0213
        """Convert the attribute's value for optimal dumping to YAML."""
        raise NotImplementedError("method must be implemented in subclasses")



"""Core YAML mapping functionality."""

import os
import functools

import yaml

from . import common
from . import settings

log = common.logger(__name__)


def readwrite(func):
    """Decorator for methods that require the file to exist."""
    @functools.wraps(func)
    def wrapped(self, *args, **kwargs):
        """Wrapped method."""
        if not self.exists:
            msg = "cannot access deleted: {}".format(self.path)
            raise common.FileError(msg)
        return func(self, *args, **kwargs)
    return wrapped


class Mapper:

    """Utility class to manipulate YAML files.

    To start mapping attributes to a file:

        create -> [empty] -> FILE

    When getting an attribute:

        FILE -> read -> [text] -> load -> [dict] -> fetch -> ATTRIBUTES

    When setting an attribute:

        ATTRIBUTES -> store -> [dict] -> dump -> [text] -> write -> FILE

    After the mapped file is no longer needed:

        delete -> [null] -> FILE

    """

    def __init__(self, path):
        self.path = path
        self.auto = False
        self.exists = os.path.isfile(self.path)
        self._activity = False
        self._timestamp = 0

    def __str__(self):
        return str(self.path)

    @property
    def _fake(self):  # pylint: disable=R0201
        """Get a string indicating the fake setting to use in logging."""
        return "(fake) " if settings.fake else ''

    def create(self, obj):
        """Create a new file for the object."""
        log.info("creating %s'%s' for %r...", self._fake, self, obj)
        if self.exists:
            log.warning("already created: %s", self)
            return
        if not self._fake:
            common.create_dirname(self.path)
            common.touch(self.path)
        self.modified = False
        self.exists = True

    @readwrite
    def fetch(self, obj, attrs, force=False):
        """Update the object's mapped attributes from its file."""
        if self._activity:
            return
        if not self.modified and not force:
            return
        self._activity = True
        log.debug("%sfetching %r from %s'%s'...", "force-" if force else "",
                  obj, self._fake, self.path)

        # Parse data from file
        if self._fake:
            text = getattr(obj, 'yorm_fake', "")
        else:
            text = self._read()
        data = self._load(text, self.path)
        log.trace("loaded: {}".format(data))

        # Update attributes
        for name, data in data.items():
            try:
                converter = attrs[name]
            except KeyError:
                # TODO: determine if this runtime import is the best way to do this
                from . import standard
                converter = standard.match(name, data)
                attrs[name] = converter
            value = converter.to_value(data)
            log.trace("value fetched: '{}' = {}".format(name, repr(value)))
            setattr(obj, name, value)

        # Set meta attributes
        self.modified = False
        self._activity = False

    @readwrite
    def _read(self):
        """Read text from the object's file.

        :param path: path to a text file

        :return: contexts of text file

        """
        return common.read_text(self.path)

    @staticmethod
    def _load(text, path):
        """Load YAML data from text.

        :param text: text read from a file
        :param path: path to the file (for displaying errors)

        :return: dictionary of YAML data

        """
        return common.load_yaml(text, path)

    @readwrite
    def store(self, obj, attrs):
        """Format and save the object's mapped attributes to its file."""
        if self._activity:
            return
        self._activity = True
        log.debug("storing %r to %s'%s'...", obj, self._fake, self.path)

        # Format the data items
        data = {}
        for name, converter in attrs.items():
            try:
                value = getattr(obj, name)
            except AttributeError as exc:
                log.debug(exc)
                value = None
            data2 = converter.to_data(value)
            log.trace("data to store: '%s' = %r", name, data2)
            data[name] = data2

        # Dump data to file
        text = self._dump(data)
        if self._fake:
            obj.yorm_fake = text
        else:
            self._write(text)

        # Set meta attributes
        self.modified = False
        self._activity = False

    @staticmethod
    def _dump(data):
        """Dump YAML data to text.

        :param data: dictionary of YAML data

        :return: text to write to a file

        """
        return yaml.dump(data, default_flow_style=False, allow_unicode=True)

    @readwrite
    def _write(self, text):
        """Write text to the object's file.

        :param text: text to write to a file
        :param path: path to the file

        """
        common.write_text(text, self.path)

    @property
    def modified(self):
        """Determine if the file has been modified."""
        if self._fake:
            log.trace("file is modified (it is fake)")
            return True
        elif not self.exists:
            log.trace("file is modified (it is deleted)")
            return True
        else:
            was = self._timestamp
            now = common.stamp(self.path)
            log.trace("file is %smodified (%s -> %s)",
                      "not " if was == now else "",
                      was, now)
            return was != now

    @modified.setter
    def modified(self, changes):
        """Mark the file as modified if there are changes."""
        if changes:
            log.trace("marked %sfile as modified", self._fake)
            self._timestamp = 0
        else:
            if self._fake:
                self._timestamp = None
            else:
                self._timestamp = common.stamp(self.path)
            log.trace("marked %sfile as not modified", self._fake)

    def delete(self):
        """Delete the object's file from the file system."""
        if self.exists:
            log.info("deleting %s'%s'...", self._fake, self.path)
            if not self._fake:
                common.delete(self.path)
            self.exists = False
        else:
            log.warning("already deleted: %s", self)



"""Converter classes for abstract container types."""

import abc

from . import common
from . import standard

log = common.logger(__name__)


class ContainerMeta(abc.ABCMeta):

    """Metaclass to initialize `yorm_attrs` on class declaration."""

    def __init__(cls, name, bases, nmspc):
        super().__init__(name, bases, nmspc)
        cls.yorm_attrs = {}


class Dictionary(dict, metaclass=ContainerMeta):

    """Base class for a dictionary of attribute converters."""

    @classmethod
    def default(cls):
        """Create an uninitialized object."""
        if cls is Dictionary:
            msg = "Dictionary class must be subclassed to use"
            raise NotImplementedError(msg)

        return dict()

    @classmethod
    def to_value(cls, obj):  # pylint: disable=E0213
        """Convert all loaded values back to its original attribute types."""
        value = cls.default()

        # Convert object attributes to a dictionary
        yorm_attrs = cls.yorm_attrs.copy()
        if isinstance(obj, cls):
            items = obj.__dict__.items()
            dictionary = {k: v for k, v in items if k in yorm_attrs}
        else:
            dictionary = cls.to_dict(obj)

        # Map object attributes to converters
        for name, data in dictionary.items():
            try:
                converter = yorm_attrs.pop(name)
            except KeyError:
                converter = standard.match(name, data, nested=True)
                cls.yorm_attrs[name] = converter
            value[name] = converter.to_value(data)

        # Create default values for unmapped converters
        for name, converter in yorm_attrs.items():
            log.trace("adding missing nested key '{}'...".format(name))
            value[name] = converter.to_value(None)

        return value

    @classmethod
    def to_data(cls, obj):  # pylint: disable=E0213
        """Convert all attribute values for optimal dumping to YAML."""
        value = cls.to_value(obj)

        data = {}

        for name, converter in cls.yorm_attrs.items():
            data[name] = converter.to_data(value.get(name, None))

        return data

    @staticmethod
    def to_dict(obj):
        """Convert a dictionary-like object to a dictionary.

        >>> Dictionary.to_dict({'key': 42})
        {'key': 42}

        >>> Dictionary.to_dict("key=42")
        {'key': '42'}

        >>> Dictionary.to_dict("key")
        {'key': None}

        >>> Dictionary.to_dict(None)
        {}

        """
        if isinstance(obj, dict):
            return obj
        elif isinstance(obj, str):
            text = obj.strip()
            parts = text.split('=')
            if len(parts) == 2:
                return {parts[0]: parts[1]}
            else:
                return {text: None}
        else:
            return {}


class List(list, metaclass=ContainerMeta):

    """Base class for a homogeneous list of attribute converters."""

    ALL = 'all'

    @classmethod
    def default(cls):
        """Create an uninitialized object."""
        if cls is List:
            raise NotImplementedError("List class must be subclassed to use")
        if not cls.item_type:
            raise NotImplementedError("List subclass must specify item type")

        return cls.__new__(cls)

    @common.classproperty
    def item_type(cls):  # pylint: disable=E0213
        """Get the converter class for all items."""
        return cls.yorm_attrs.get(cls.ALL)

    @classmethod
    def to_value(cls, obj):  # pylint: disable=E0213
        """Convert all loaded values back to the original attribute type."""
        value = cls.default()

        for item in cls.to_list(obj):
            value.append(cls.item_type.to_value(item))

        return value

    @classmethod
    def to_data(cls, obj):  # pylint: disable=E0213
        """Convert all attribute values for optimal dumping to YAML."""
        value = cls.to_value(obj)

        data = []

        for item in value:
            data.append(cls.item_type.to_data(item))

        return data

    @staticmethod
    def to_list(obj):
        """Convert a list-like object to a list.

        >>> List.to_list([1, 2, 3])
        [1, 2, 3]

        >>> List.to_list("a,b,c")
        ['a', 'b', 'c']

        >>> List.to_list("item")
        ['item']

        >>> List.to_list(None)
        []

        """
        if isinstance(obj, list):
            return obj
        elif isinstance(obj, str):
            text = obj.strip()
            if ',' in text and ' ' not in text:
                return text.split(',')
            else:
                return text.split()
        elif obj is not None:
            return [obj]
        else:
            return []



#!/usr/bin/env python
# pylint:disable=R0201

"""Integration tests for the `yorm.settings.fake` option."""

import os
import pytest
from unittest.mock import patch

import yorm

from . import strip


@patch('yorm.settings.fake', True)
class TestFake:

    """Integration tests with `yorm.fake` enabled."""

    @yorm.attr(value=yorm.standard.Integer)
    @yorm.sync("path/to/{self.name}.yml")
    class Sample:

        """Sample class for fake mapping."""

        def __init__(self, name):
            self.name = name
            self.value = 0

    def test_fake(self, tmpdir):
        """Verify no file is created with fake enabled."""
        tmpdir.chdir()
        sample = self.Sample('sample')

        # ensure no file is created
        assert "path/to/sample.yml" == sample.yorm_path
        assert not os.path.exists(sample.yorm_path)

        # change object values
        sample.value = 42

        # check fake file
        assert strip("""
        value: 42
        """) == sample.yorm_fake

        # ensure no file is created
        assert not os.path.exists(sample.yorm_path)

        # change fake file
        text = strip("""
        value2: abc
        """)
        setattr(sample, 'yorm_fake', text)

        # check object values
        assert "abc" == sample.value2

        # ensure no file is created
        assert not os.path.exists(sample.yorm_path)


if __name__ == '__main__':
    pytest.main()
